#' Plot Fastest Lap
#'
#' @description Creates a ggplot graphic that details the fastest lap for a driver in a race.
#' Complete with a gearshift or speed analysis.
#'
#' @param season number from 2018 to current season (defaults to current season).
#' @param race number from 1 to 23 (depending on season selected) and defaults
#' to most recent.
#' @param round number from 1 to 23 (depending on season selected) and defaults
#' to most recent.
#' @param session the code for the session to load Options are `'FP1'`, `'FP2'`, `'FP3'`,
#' `'Q'`, `'S'`, `'SS'`, and `'R'`. Default is `'R'`, which refers to Race.
#' @param driver three letter driver code (see load_drivers() for a list).
#' @param color argument that indicates which variable to plot along the
#' circuit. Choice of `'gear'` or `'speed'`, default `'gear'`.
#' @importFrom magrittr "%>%"
#' @importFrom rlang .data
#' @return A ggplot object that indicates grand prix, driver, time and selected
#' color variable.
#' @export
#' @examples
#' # Plot Verstappen's fastest lap (speed) from Bahrain 2023:
#' if (interactive()) {
#'   plot_fastest(2023, 1, "R", "VER", "speed")
#' }
plot_fastest <- function(season = get_current_season(), round = 1, session = "R", driver, color = "gear",
                         race = lifecycle::deprecated()) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    cli::cli_abort("f1dataR::plot_fastest() requires ggplot2 package installation")
  }
  if (lifecycle::is_present(race)) {
    lifecycle::deprecate_warn("1.0.0", "plot_fastest(race)", "plot_fastest(round)")
    round <- race
  }

  cli::cli_alert_info("If the session has not been loaded yet, this could take a minute\n\n")

  driver_data <- load_driver_telemetry(season, round, session, driver, "fastest") %>%
    dplyr::mutate(
      x = .data$x - mean(range(.data$x, na.rm = TRUE)),
      y = .data$y - mean(range(.data$y, na.rm = TRUE))
    )

  selected_driver_id <- load_drivers(season) %>%
    dplyr::filter(.data$code == driver) %>%
    dplyr::pull(.data$driver_id)

  lap_time <- load_laps(season, round) %>%
    dplyr::filter(.data$driver_id == selected_driver_id) %>%
    dplyr::filter(.data$time_sec == min(.data$time_sec)) %>%
    dplyr::pull(.data$time)

  rnd <- round
  # I can't figure out why but for some reason the filter .data$round == round doesn't work. Rename to rnd fixes.
  race_name <- load_schedule(season) %>%
    dplyr::filter(.data$round == rnd) %>%
    dplyr::pull(.data$race_name)

  if (!(session %in% c("r", "R"))) {
    race_name <- dplyr::case_match(
      session,
      c("q", "Q") ~ paste0(race_name, " Qualifying"),
      c("s", "S") ~ paste0(race_name, " Sprint"),
      c("fp1", "FP1") ~ paste0(race_name, " FP1"),
      c("fp2", "FP2") ~ paste0(race_name, " FP2"),
      c("fp3", "FP3") ~ paste0(race_name, " FP3")
    )
  }

  lim_min <- min(min(driver_data$x, na.rm = TRUE), min(driver_data$y, na.rm = TRUE))
  lim_max <- max(max(driver_data$x, na.rm = TRUE), max(driver_data$y, na.rm = TRUE))
  if (color == "gear") {
    ggplot2::ggplot(driver_data, ggplot2::aes(.data$x, .data$y, color = as.factor(.data$n_gear), group = 1)) +
      ggplot2::geom_path(linewidth = 4, lineend = "round") +
      ggplot2::scale_color_manual(
        name = "Gear",
        values = c("#BC3C29", "#0072B5", "#E18727", "#20854E", "#7876B1", "#6F99AD", "#FFDC91", "#EE4C97"),
        aesthetics = c("color", "fill")
      ) +
      theme_dark_f1() +
      ggplot2::scale_x_continuous(limits = c(lim_min, lim_max)) +
      ggplot2::scale_y_continuous(limits = c(lim_min, lim_max)) +
      ggplot2::labs(
        title = glue::glue("{year} {race_name}", year = season, race = race_name),
        subtitle = glue::glue("{driver} Fastest Lap | {lap_time}", driver = driver, lap_time = lap_time),
        caption = "Generated by {f1dataR} package"
      )
  } else if (color == "speed") {
    ggplot2::ggplot(driver_data, ggplot2::aes(.data$x, .data$y, color = .data$speed, group = 1)) +
      ggplot2::geom_path(linewidth = 4, lineend = "round") +
      ggplot2::scale_color_gradient(low = "white", high = "red") +
      theme_dark_f1() +
      ggplot2::scale_x_continuous(limits = c(lim_min, lim_max)) +
      ggplot2::scale_y_continuous(limits = c(lim_min, lim_max)) +
      ggplot2::labs(
        title = glue::glue("{year} {race_name}", year = season, race = race_name),
        subtitle = glue::glue("{driver} Fastest Lap | {lap_time}", driver = driver, lap_time = lap_time),
        caption = "Generated by {f1dataR} package"
      )
  }
}
